@@ -166,46 +166,7 @@ class SmappeeMqtt(threading.Thread):
 
             # smart device and ETC topics
             elif message.topic.startswith(f'{self.topic_prefix}/etc/'):
-                if message.topic.endswith('/devices'):
-                    devices = json.loads(message.payload)
-                    for device in devices:
-                        self._service_location._add_smart_device(
-                            uuid=device.get('uuid'),
-                            name=device.get('name'),
-                            category=device.get('category'),
-                            implementation=device.get('implementation'),
-                            minCurrent=device.get('minimumCurrent'),
-                            maxCurrent=device.get('maximumCurrent'),
-                            measurements=device.get('measurements'),
-                        )
-                elif message.topic.endswith('/devices/updated'):
-                    pass
-                elif message.topic.endswith('/action/setcurrent'):
-                    smart_device_uuid = message.topic.split('/')[-3]
-                    set_current_details = json.loads(message.payload)
-                    self._service_location.smart_devices[smart_device_uuid].set_current(
-                        phase=set_current_details.get('phase'),
-                        current=set_current_details.get('value')
-                    )
-                elif message.topic.endswith('/action/startcharging'):
-                    pass
-                elif message.topic.endswith('/action/stopcharging'):
-                    pass
-                elif message.topic.endswith('/action/smartcharging'):
-                    pass
-                elif message.topic.endswith('/state'):
-                    details = json.loads(message.payload)
-                    smart_device_uuid = details.get('deviceUUID')
-                    connection_status = details.get('connectionStatus')
-                    if smart_device_uuid in self._service_location.smart_devices:
-                        self._service_location.smart_devices[smart_device_uuid].set_connection_status(connection_status=connection_status)
-                elif message.topic.endswith('/etc/measuredvalues'):
-                    pass
-                elif message.topic.endswith('/property/chargingstate'):
-                    pass
-
-                elif config['MQTT']['discovery']:
-                    print(message.topic, message.payload)
+                pass
 
             # specific HASS.io topics
             elif message.topic == f'{self.topic_prefix}/homeassistant/event':
@@ -272,14 +233,24 @@ class SmappeeLocalMqtt(threading.Thread):
 
     def __init__(self, serial_number):
         self._client = None
+        self.service_location = None
         self._serial_number = serial_number
+        self._service_location_id = None
         self._service_location_uuid = None
         threading.Thread.__init__(
             self,
             name=f'SmappeeLocalMqttListener_{self._serial_number}'
         )
+
         self.realtime = {}
         self.phase_type = None
+        self.measurements = {}
+
+        self.switch_sensors = []
+        self.smart_plugs = []
+        self.actuators_connection_state = {}
+        self.actuators_state = {}
+
         self._timezone = None
 
     @property
@@ -300,76 +271,120 @@ class SmappeeLocalMqtt(threading.Thread):
             # realtime local power values
             if message.topic.endswith('/realtime'):
                 self.realtime = json.loads(message.payload)
+                if self.service_location is not None:
+                    self.service_location._update_realtime_data(realtime_data=self.realtime)
+
             elif message.topic.endswith('/config'):
                 c = json.loads(message.payload)
                 self._timezone = c.get('timeZone')
-                self._serviceLocationId = c.get('serviceLocationId')
-                self._serviceLocationUuid = c.get('serviceLocationUid')
+                self._service_location_id = c.get('serviceLocationId')
+                self._service_location_uuid = c.get('serviceLocationUuid')
+            elif message.topic.endswith('channelConfig'):
+                pass
             elif message.topic.endswith('/channelConfigV2'):
                 self._channel_config = json.loads(message.payload)
                 self.phase_type = self._channel_config.get('dataProcessingSpecification', {}).get('phaseType', None)
+
+                # extract measurements from channelConfigV2
+                measurements_dict = {}
+                for m in self._channel_config.get('dataProcessingSpecification', {}).get('measurements', []):
+                    if m.get('flow') == 'CONSUMPTION' and m.get('connectionType') == 'SUBMETER':
+                        if not m['name'] in measurements_dict.keys():
+                            measurements_dict[m['name']] = []
+                        measurements_dict[m['name']].append(m['publishIndex'])
+                    elif m.get('flow') == 'CONSUMPTION' and m.get('connectionType') == 'GRID':
+                        if not 'Grid' in measurements_dict.keys():
+                            measurements_dict['Grid'] = []
+                        measurements_dict['Grid'].append(m['publishIndex'])
+                    elif m.get('flow') == 'PRODUCTION' and m.get('connectionType') == 'GRID':
+                        if not 'Solar' in measurements_dict.keys():
+                            measurements_dict['Solar'] = []
+                        measurements_dict['Solar'].append(m['publishIndex'])
+
+                self.measurements = {}
+                for m_name, m_index in measurements_dict.items():
+                    self.measurements[m_name] = list(set(m_index))
+
             elif message.topic.endswith('/sensorConfig'):
                 pass
             elif message.topic.endswith('/homeControlConfig'):
-                self._home_control_config = json.loads(message.payload)
+                # switches
+                switches = json.loads(message.payload).get('switchActuators', [])
+                for switch in switches:
+                    self.switch_sensors.append({
+                        'nodeId': switch['nodeId'],
+                        'name': switch['name'],
+                        'serialNumber': switch['serialNumber']
+                    })
+
+                # plugs
+                plugs = json.loads(message.payload).get('smartplugActuators', [])
+                for plug in plugs:
+                    self.smart_plugs.append({
+                        'nodeId': plug['nodeId'],
+                        'name': plug['name']
+                    })
             elif message.topic.endswith('/presence'):
                 pass
             elif message.topic.endswith('/aggregated'):
                 pass
+            elif message.topic.endswith('/aggregatedSwitch'):
+                pass
+            elif message.topic.endswith('/etc/measuredvalues'):
+                pass
+            elif message.topic.endswith('/networkstatistics'):
+                pass
+            elif message.topic.endswith('/scheduler'):
+                pass
+            elif message.topic.endswith('/devices'):
+                pass
+            elif message.topic.endswith('/action/setcurrent'):
+                pass
+            elif message.topic.endswith('/trigger'):
+                pass
+            elif message.topic.endswith('/connectionState'):
+                actuator_id = int(message.topic.split('/')[-2])
+                self.actuators_connection_state[actuator_id] = json.loads(message.payload).get('value')
+            elif message.topic.endswith('/state'):
+                actuator_id = int(message.topic.split('/')[-2])
+                self.actuators_state[actuator_id] = json.loads(message.payload).get('value')
+
+                if self.service_location is not None:
+                    self.service_location.set_actuator_state(
+                        id=actuator_id,
+                        state='{0}_{0}'.format(self.actuators_state[actuator_id]),
+                        api=False
+                    )
+            elif message.topic.endswith('/setstate'):
+                actuator_id = int(message.topic.split('/')[-2])
+                p = str(message.payload.decode('utf-8')).replace("\'", "\"")
+                self.actuators_state[actuator_id] = json.loads(p).get('value')
             else:
                 print('Processing MQTT message from topic {0} with value {1}'.format(message.topic, message.payload))
 
         except Exception:
             traceback.print_exc()
-            #
-            # if message.topic == f'{self.topic_prefix}/config':
-            #     config_details = json.loads(message.payload)
-            #     self._service_location.firmware_version = config_details.get('firmwareVersion')
-            #     self._service_location._service_location_uuid = config_details.get('serviceLocationUuid')
-            #     self._service_location._service_location_id = config_details.get('serviceLocationId')
-            # elif message.topic == f'{self.topic_prefix}/sensorConfig':
-            #     pass
-            # elif message.topic == f'{self.topic_prefix}/homeControlConfig':
-            #     pass
-            #
-            # # controllable nodes (general messages)
-            # elif message.topic == f'{self.topic_prefix}':
-            #     msg = json.loads(message.payload)
-            #
-            #     # turn ON/OFF comfort plug
-            #     if msg.get('messageType') == 1283:
-            #         id = msg['content']['controllableNodeId']
-            #         plug_state = msg['content']['action']
-            #         plug_state_since = int(msg['content']['timestamp'] / 1000)
-            #         self._service_location.set_actuator_state(id=id,
-            #                                                   state=plug_state,
-            #                                                   since=plug_state_since,
-            #                                                   api=False)
-            #
-            # # actuator topics
-            # elif message.topic.startswith(f'{self.topic_prefix}/plug/'):
-            #     plug_id = int(message.topic.split('/')[-2])
-            #     payload = json.loads(message.payload)
-            #     plug_state, plug_state_since = payload.get('value'), payload.get('since')
-            #
-            #     state_type = message.topic.split('/')[-1]
-            #     if state_type == 'state' and self._kind == 'central':  # todo: remove and condition
-            #         self._service_location.set_actuator_state(id=plug_id,
-            #                                                   state=plug_state,
-            #                                                   since=plug_state_since,
-            #                                                   api=False)
-            #     elif state_type == 'connectionState':
-            #         self._service_location.set_actuator_connection_state(id=plug_id,
-            #                                                              connection_state=plug_state,
-            #
 
-    def is_config_ready(self, timeout=60):
+    def set_actuator_state(self, service_location_id, actuator_id, state_id):
+        state = None
+        if state_id == 'ON_ON':
+            state = 'ON'
+        elif state_id == 'OFF_OFF':
+            state = 'OFF'
+
+        if state is not None:
+            self._client.publish(
+                topic="servicelocation/{0}/plug/{1}/setstate".format(self._service_location_uuid, actuator_id),
+                payload=json.dumps({"value": state})
+            )
+
+    def is_config_ready(self, timeout=60, interval=5):
         c = 0
         while c < timeout:
             if self.phase_type is not None:
                 break
-            c += 1
-            time.sleep(1)
+            c += interval
+            time.sleep(interval)
 
     def start_attempt(self):
         client = mqtt.Client(client_id='smappeeLocalMqttConnectionAttempt')
